﻿#pragma TextEncoding="UTF-8"
#pragma rtGlobals=3
#pragma DefaultTab={3,20,4}
#pragma version=1.60
#pragma IndependentModule=ScrollToZoom
#pragma IgorVersion=9

// Project Updater header
static constant kProjectID = 22498 // the project node on IgorExchange
static strconstant ksShortTitle = "ScrollToZoom" // the project short title on IgorExchange

#define global

// key codes
// 2: shift; 4: option/alt; 8: command/ctrl
// On Windows, alt key is reserved for graph drag.
static constant kZoomKey   =  8
static constant kFasterKey =  2 // 0 to disable
static constant kZoomSpeed = 70
static constant kReverse   =  0 // 1 to reverse the direction of expansion

#ifdef global

static function AfterWindowCreatedHook(string win, variable type)
	if (type == 1)
		SetWindow $win hook(hScrollToZoom)=ScrollToZoom#hookScrollToZoom
	endif
	return 0
end

static function AfterFileOpenHook(variable refNum, string fileNameStr, string pathNameStr, string fileTypeStr, string fileCreatorStr, variable fileKind)
	if (filekind == 1 || filekind == 2) // opened an experiment
		SetHooks(1)
	endif
	return 0
end

static function SetHooks(int activate)
	wave/T wGraphs = ListToTextWave(WinList("*",";","WIN:1"), ";")
	for (string strWin : wGraphs)
		if (activate)
			SetWindow $strWin hook(hScrollToZoom)=ScrollToZoom#hookScrollToZoom
		else
			SetWindow $strWin hook(hScrollToZoom)=$""
		endif
	endfor
end

#endif

menu "Graph", dynamic
	ScrollToZoom#zoomMenu(), /Q, ScrollToZoom#toggleZoom()
end

menu "Panel", dynamic
	ScrollToZoom#zoomMenu(), /Q, ScrollToZoom#toggleZoom()
end

function/S zoomMenu()
	string strParent = ParseFilePath(0, WinName(0, 65), "#", 0, 0)
	if (!strlen(strParent))
		return ""
	endif
	GetWindow/Z $strParent hook(hScrollToZoom)
	return SelectString(strlen(s_value)>0, "", "!" + num2char(18)) + "Scroll-to-Zoom"
end

function toggleZoom()
	string strParent = ParseFilePath(0, WinName(0, 65), "#", 0, 0)
	if (strlen(strParent))
		GetWindow $strParent hook(hScrollToZoom)
		SetWindow $strParent hook(hScrollToZoom) = $SelectString(strlen(s_value)>0, "ScrollToZoom#hookScrollToZoom", "")
	endif
end

function hookScrollToZoom(STRUCT WMWinHookStruct &s)
	
	if (WinType(s.winName) != 1)
		return 0
	endif
	
	if (s.eventCode == 22 && s.eventMod&kZoomKey) // mousewheel/touchpad + zoom key

		string strAxes = AxisList(s.WinName), axis = "", type = "", info = ""
		int i, isHorizontal, isLog
		Make/D/free/N=3 wAx // free wave to hold axis minimum, maximum, axis value for mouse location
		variable expansion = 1 + sign(kReverse - 1) * s.wheelDy * kZoomSpeed / 5000
		if (s.eventMod & kFasterKey)
			if (kFasterKey==2 && s.wheelDy==0)
				// this works when shift key also transforms wheel.dY to wheel.dX
				s.wheelDy = s.wheelDx
			endif
			expansion = 1 + sign(kReverse - 1) * s.wheelDy * kZoomSpeed / 500
		endif
				
		for (i=ItemsInList(strAxes);i>=0;i--)
			axis = StringFromList(i, strAxes)
			info = AxisInfo(s.WinName, axis)
			type = StringByKey("AXTYPE", info)
			isHorizontal = (cmpstr(type, "bottom")==0 || cmpstr(type,"top")==0)
			isHorizontal = isHorizontal || (cmpstr(type[0,2],"/B=")==0 || cmpstr(type[0,2],"/T=")==0)
			isLog = (NumberByKey("log(x)", info, "="))
					
			GetAxis/W=$s.WinName/Q $axis
			wAx = {v_min, v_max, AxisValFromPixel(s.WinName, axis, isHorizontal ? s.mouseLoc.h : s.mouseLoc.v)}
			if (WaveMax(wAx) == wAx[2] || WaveMin(wAx) == wAx[2])
				continue
			endif
						
			if (isLog)
				wAx = log(wAx)
				wAx = wAx[2] - (wAx[2] - wAx[p]) * expansion
				wAx = alog(wAx)
			else
				wAx = wAx[2] - (wAx[2] - wAx[p]) * expansion
			endif
					
			WaveStats/Q/M=1 wAx
			if ( (V_numNaNs+V_numInfs) || wAx[1]==wAx[0] )
				continue
			endif
			
			if (wAx[1] > wAx[0])
				SetAxis/W=$s.WinName $axis, wAx[0], wAx[1]
			else
				SetAxis/R/W=$s.WinName $axis, wAx[0], wAx[1]
			endif
		endfor
	endif
	return 0
end